@@ -15,23 +15,29 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! A shared HTTP client implementation incorporating retries
+//! [`RetryConfig`] connection retry policy
 
 use crate::client::backoff::{Backoff, BackoffConfig};
 use crate::client::builder::HttpRequestBuilder;
-use crate::client::connection::HttpErrorKind;
-use crate::client::{HttpClient, HttpError, HttpRequest, HttpResponse};
+use crate::client::{HttpClient, HttpError, HttpErrorKind, HttpRequest, HttpResponse};
 use crate::PutPayload;
 use futures::future::BoxFuture;
 use http::{Method, Uri};
 use reqwest::header::LOCATION;
 use reqwest::StatusCode;
+#[cfg(not(all(target_arch = "wasm32", target_os = "unknown")))]
 use std::time::{Duration, Instant};
 use tracing::info;
+#[cfg(all(target_arch = "wasm32", target_os = "unknown"))]
+use web_time::{Duration, Instant};
 
 /// Retry request error
-#[derive(Debug, thiserror::Error)]
-pub struct RetryError {
+#[derive(Debug)]
+pub struct RetryError(Box<RetryErrorImpl>);
+
+/// Box error to avoid large error variant
+#[derive(Debug)]
+struct RetryErrorImpl {
     method: Method,
     uri: Option<Uri>,
     retries: usize,
@@ -43,48 +49,59 @@ pub struct RetryError {
 
 impl std::fmt::Display for RetryError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "Error performing {} ", self.method)?;
-        match &self.uri {
+        write!(f, "Error performing {} ", self.0.method)?;
+        match &self.0.uri {
             Some(uri) => write!(f, "{uri} ")?,
             None => write!(f, "REDACTED ")?,
         }
-        write!(f, "in {:?}", self.elapsed)?;
-        if self.retries != 0 {
+        write!(f, "in {:?}", self.0.elapsed)?;
+        if self.0.retries != 0 {
             write!(
                 f,
                 ", after {} retries, max_retries: {}, retry_timeout: {:?} ",
-                self.retries, self.max_retries, self.retry_timeout
+                self.0.retries, self.0.max_retries, self.0.retry_timeout
             )?;
         }
-        write!(f, " - {}", self.inner)
+        write!(f, " - {}", self.0.inner)
+    }
+}
+
+impl std::error::Error for RetryError {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+        Some(&self.0.inner)
     }
 }
 
 /// Context of the retry loop
-struct RetryContext {
-    method: Method,
-    uri: Option<Uri>,
+///
+/// Most use-cases should use [`RetryExt`] and [`RetryableRequestBuilder`], however,
+/// [`RetryContext`] allows preserving retry state across multiple [`RetryableRequest`]
+pub(crate) struct RetryContext {
+    backoff: Backoff,
     retries: usize,
     max_retries: usize,
-    start: Instant,
     retry_timeout: Duration,
+    start: Instant,
 }
 
 impl RetryContext {
-    fn err(self, error: RequestError) -> RetryError {
-        RetryError {
-            uri: self.uri,
-            method: self.method,
-            retries: self.retries,
-            max_retries: self.max_retries,
-            elapsed: self.start.elapsed(),
-            retry_timeout: self.retry_timeout,
-            inner: error,
+    pub(crate) fn new(config: &RetryConfig) -> Self {
+        Self {
+            max_retries: config.max_retries,
+            retry_timeout: config.retry_timeout,
+            backoff: Backoff::new(&config.backoff),
+            retries: 0,
+            start: Instant::now(),
         }
     }
 
-    fn exhausted(&self) -> bool {
-        self.retries == self.max_retries || self.start.elapsed() > self.retry_timeout
+    pub(crate) fn exhausted(&self) -> bool {
+        self.retries >= self.max_retries || self.start.elapsed() > self.retry_timeout
+    }
+
+    pub(crate) fn backoff(&mut self) -> Duration {
+        self.retries += 1;
+        self.backoff.next()
     }
 }
 
@@ -111,12 +128,12 @@ pub enum RequestError {
 impl RetryError {
     /// Returns the underlying [`RequestError`]
     pub fn inner(&self) -> &RequestError {
-        &self.inner
+        &self.0.inner
     }
 
     /// Returns the status code associated with this error if any
     pub fn status(&self) -> Option<StatusCode> {
-        match &self.inner {
+        match self.inner() {
             RequestError::Status { status, .. } | RequestError::Response { status, .. } => {
                 Some(*status)
             }
@@ -126,7 +143,7 @@ impl RetryError {
 
     /// Returns the error body if any
     pub fn body(&self) -> Option<&str> {
-        match &self.inner {
+        match self.inner() {
             RequestError::Status { body, .. } => body.as_deref(),
             RequestError::Response { body, .. } => Some(body),
             RequestError::BareRedirect | RequestError::Http(_) => None,
@@ -176,7 +193,7 @@ impl From<RetryError> for std::io::Error {
             Some(StatusCode::UNAUTHORIZED) | Some(StatusCode::FORBIDDEN) => {
                 ErrorKind::PermissionDenied
             }
-            _ => match &err.inner {
+            _ => match err.inner() {
                 RequestError::Http(h) => match h.kind() {
                     HttpErrorKind::Timeout => ErrorKind::TimedOut,
                     HttpErrorKind::Connect => ErrorKind::NotConnected,
@@ -242,13 +259,59 @@ fn body_contains_error(response_body: &str) -> bool {
     response_body.contains("InternalError") || response_body.contains("SlowDown")
 }
 
+/// Combines a [`RetryableRequest`] with a [`RetryContext`]
+pub(crate) struct RetryableRequestBuilder {
+    request: RetryableRequest,
+    context: RetryContext,
+}
+
+impl RetryableRequestBuilder {
+    /// Set whether this request is idempotent
+    ///
+    /// An idempotent request will be retried on timeout even if the request
+    /// method is not [safe](https://datatracker.ietf.org/doc/html/rfc7231#section-4.2.1)
+    pub(crate) fn idempotent(mut self, idempotent: bool) -> Self {
+        self.request.idempotent = Some(idempotent);
+        self
+    }
+
+    /// Set whether this request should be retried on a 409 Conflict response.
+    #[cfg(feature = "aws")]
+    pub(crate) fn retry_on_conflict(mut self, retry_on_conflict: bool) -> Self {
+        self.request.retry_on_conflict = retry_on_conflict;
+        self
+    }
+
+    /// Set whether this request contains sensitive data
+    ///
+    /// This will avoid printing out the URL in error messages
+    #[allow(unused)]
+    pub(crate) fn sensitive(mut self, sensitive: bool) -> Self {
+        self.request.sensitive = sensitive;
+        self
+    }
+
+    /// Provide a [`PutPayload`]
+    pub(crate) fn payload(mut self, payload: Option<PutPayload>) -> Self {
+        self.request.payload = payload;
+        self
+    }
+
+    #[allow(unused)]
+    pub(crate) fn retry_error_body(mut self, retry_error_body: bool) -> Self {
+        self.request.retry_error_body = retry_error_body;
+        self
+    }
+
+    pub(crate) async fn send(mut self) -> Result<HttpResponse> {
+        self.request.send(&mut self.context).await
+    }
+}
+
+/// A retryable request
 pub(crate) struct RetryableRequest {
     client: HttpClient,
-    request: HttpRequest,
-
-    max_retries: usize,
-    retry_timeout: Duration,
-    backoff: Backoff,
+    http: HttpRequest,
 
     sensitive: bool,
     idempotent: Option<bool>,
@@ -259,64 +322,26 @@ pub(crate) struct RetryableRequest {
 }
 
 impl RetryableRequest {
-    /// Set whether this request is idempotent
-    ///
-    /// An idempotent request will be retried on timeout even if the request
-    /// method is not [safe](https://datatracker.ietf.org/doc/html/rfc7231#section-4.2.1)
-    pub(crate) fn idempotent(self, idempotent: bool) -> Self {
-        Self {
-            idempotent: Some(idempotent),
-            ..self
-        }
-    }
-
-    /// Set whether this request should be retried on a 409 Conflict response.
-    #[cfg(feature = "aws")]
-    pub(crate) fn retry_on_conflict(self, retry_on_conflict: bool) -> Self {
-        Self {
-            retry_on_conflict,
-            ..self
-        }
-    }
-
-    /// Set whether this request contains sensitive data
-    ///
-    /// This will avoid printing out the URL in error messages
     #[allow(unused)]
     pub(crate) fn sensitive(self, sensitive: bool) -> Self {
         Self { sensitive, ..self }
     }
 
-    /// Provide a [`PutPayload`]
-    pub(crate) fn payload(self, payload: Option<PutPayload>) -> Self {
-        Self { payload, ..self }
+    fn err(&self, error: RequestError, ctx: &RetryContext) -> RetryError {
+        RetryError(Box::new(RetryErrorImpl {
+            uri: (!self.sensitive).then(|| self.http.uri().clone()),
+            method: self.http.method().clone(),
+            retries: ctx.retries,
+            max_retries: ctx.max_retries,
+            elapsed: ctx.start.elapsed(),
+            retry_timeout: ctx.retry_timeout,
+            inner: error,
+        }))
     }
 
-    #[allow(unused)]
-    pub(crate) fn retry_error_body(self, retry_error_body: bool) -> Self {
-        Self {
-            retry_error_body,
-            ..self
-        }
-    }
-
-    pub(crate) async fn send(self) -> Result<HttpResponse> {
-        let mut ctx = RetryContext {
-            retries: 0,
-            uri: (!self.sensitive).then(|| self.request.uri().clone()),
-            method: self.request.method().clone(),
-            max_retries: self.max_retries,
-            start: Instant::now(),
-            retry_timeout: self.retry_timeout,
-        };
-
-        let mut backoff = self.backoff;
-        let is_idempotent = self
-            .idempotent
-            .unwrap_or_else(|| self.request.method().is_safe());
-
+    pub(crate) async fn send(self, ctx: &mut RetryContext) -> Result<HttpResponse> {
         loop {
-            let mut request = self.request.clone();
+            let mut request = self.http.clone();
 
             if let Some(payload) = &self.payload {
                 *request.body_mut() = payload.clone().into();
@@ -337,7 +362,7 @@ impl RetryableRequest {
                         let (parts, body) = r.into_parts();
                         let body = match body.text().await {
                             Ok(body) => body,
-                            Err(e) => return Err(ctx.err(RequestError::Http(e))),
+                            Err(e) => return Err(self.err(RequestError::Http(e), ctx)),
                         };
 
                         if !body_contains_error(&body) {
@@ -346,11 +371,10 @@ impl RetryableRequest {
                         } else {
                             // Retry as if this was a 5xx response
                             if ctx.exhausted() {
-                                return Err(ctx.err(RequestError::Response { body, status }));
+                                return Err(self.err(RequestError::Response { body, status }, ctx));
                             }
 
-                            let sleep = backoff.next();
-                            ctx.retries += 1;
+                            let sleep = ctx.backoff();
                             info!(
                                 "Encountered a response status of {} but body contains Error, backing off for {} seconds, retry {} of {}",
                                 status,
@@ -361,20 +385,25 @@ impl RetryableRequest {
                             tokio::time::sleep(sleep).await;
                         }
                     } else if status == StatusCode::NOT_MODIFIED {
-                        return Err(ctx.err(RequestError::Status { status, body: None }));
+                        return Err(self.err(RequestError::Status { status, body: None }, ctx));
                     } else if status.is_redirection() {
                         let is_bare_redirect = !r.headers().contains_key(LOCATION);
                         return match is_bare_redirect {
-                            true => Err(ctx.err(RequestError::BareRedirect)),
-                            false => Err(ctx.err(RequestError::Status {
-                                body: None,
-                                status: r.status(),
-                            })),
+                            true => Err(self.err(RequestError::BareRedirect, ctx)),
+                            false => Err(self.err(
+                                RequestError::Status {
+                                    body: None,
+                                    status: r.status(),
+                                },
+                                ctx,
+                            )),
                         };
                     } else {
                         let status = r.status();
                         if ctx.exhausted()
                             || !(status.is_server_error()
+                                || status == StatusCode::TOO_MANY_REQUESTS
+                                || status == StatusCode::REQUEST_TIMEOUT
                                 || (self.retry_on_conflict && status == StatusCode::CONFLICT))
                         {
                             let source = match status.is_client_error() {
@@ -387,13 +416,13 @@ impl RetryableRequest {
                                 },
                                 false => RequestError::Status { status, body: None },
                             };
-                            return Err(ctx.err(source));
+                            return Err(self.err(source, ctx));
                         };
 
-                        let sleep = backoff.next();
-                        ctx.retries += 1;
+                        let sleep = ctx.backoff();
                         info!(
-                            "Encountered server error, backing off for {} seconds, retry {} of {}",
+                            "Encountered server error with status {}, backing off for {} seconds, retry {} of {}",
+                            status,
                             sleep.as_secs_f32(),
                             ctx.retries,
                             ctx.max_retries,
@@ -402,7 +431,9 @@ impl RetryableRequest {
                     }
                 }
                 Err(e) => {
-                    // let e = sanitize_err(e);
+                    let is_idempotent = self
+                        .idempotent
+                        .unwrap_or_else(|| self.http.method().is_safe());
 
                     let do_retry = match e.kind() {
                         HttpErrorKind::Connect | HttpErrorKind::Request => true, // Request not sent, can retry
@@ -410,16 +441,13 @@ impl RetryableRequest {
                         HttpErrorKind::Unknown | HttpErrorKind::Decode => false,
                     };
 
-                    if ctx.retries == ctx.max_retries
-                        || ctx.start.elapsed() > ctx.retry_timeout
-                        || !do_retry
-                    {
-                        return Err(ctx.err(RequestError::Http(e)));
+                    if ctx.exhausted() || !do_retry {
+                        return Err(self.err(RequestError::Http(e), ctx));
                     }
-                    let sleep = backoff.next();
-                    ctx.retries += 1;
+                    let sleep = ctx.backoff();
                     info!(
-                        "Encountered transport error backing off for {} seconds, retry {} of {}: {}",
+                        "Encountered transport error of kind {:?}, backing off for {} seconds, retry {} of {}: {}",
+                        e.kind(),
                         sleep.as_secs_f32(),
                         ctx.retries,
                         ctx.max_retries,
@@ -433,8 +461,11 @@ impl RetryableRequest {
 }
 
 pub(crate) trait RetryExt {
+    /// Return a [`RetryableRequestBuilder`]
+    fn retryable(self, config: &RetryConfig) -> RetryableRequestBuilder;
+
     /// Return a [`RetryableRequest`]
-    fn retryable(self, config: &RetryConfig) -> RetryableRequest;
+    fn retryable_request(self) -> RetryableRequest;
 
     /// Dispatch a request with the given retry configuration
     ///
@@ -445,16 +476,20 @@ pub(crate) trait RetryExt {
 }
 
 impl RetryExt for HttpRequestBuilder {
-    fn retryable(self, config: &RetryConfig) -> RetryableRequest {
+    fn retryable(self, config: &RetryConfig) -> RetryableRequestBuilder {
+        RetryableRequestBuilder {
+            request: self.retryable_request(),
+            context: RetryContext::new(config),
+        }
+    }
+
+    fn retryable_request(self) -> RetryableRequest {
         let (client, request) = self.into_parts();
         let request = request.expect("request must be valid");
 
         RetryableRequest {
             client,
-            request,
-            max_retries: config.max_retries,
-            retry_timeout: config.retry_timeout,
-            backoff: Backoff::new(&config.backoff),
+            http: request,
             idempotent: None,
             payload: None,
             sensitive: false,
@@ -469,16 +504,25 @@ impl RetryExt for HttpRequestBuilder {
     }
 }
 
+#[cfg(not(target_arch = "wasm32"))]
 #[cfg(test)]
 mod tests {
     use crate::client::mock_server::MockServer;
-    use crate::client::retry::{body_contains_error, RequestError, RetryExt};
-    use crate::client::HttpClient;
+    use crate::client::retry::{body_contains_error, RequestError, RetryContext, RetryExt};
+    use crate::client::{HttpClient, HttpResponse};
     use crate::RetryConfig;
+    use http::StatusCode;
     use hyper::header::LOCATION;
+    use hyper::server::conn::http1;
+    use hyper::service::service_fn;
     use hyper::Response;
-    use reqwest::{Client, Method, StatusCode};
+    use hyper_util::rt::TokioIo;
+    use reqwest::{Client, Method};
+    use std::convert::Infallible;
+    use std::error::Error;
     use std::time::Duration;
+    use tokio::net::TcpListener;
+    use tokio::time::timeout;
 
     #[test]
     fn test_body_contains_error() {
@@ -517,7 +561,7 @@ mod tests {
         let r = do_request().await.unwrap();
         assert_eq!(r.status(), StatusCode::OK);
 
-        // Returns client errors immediately with status message
+        // Returns client errors immediately with a status message
         mock.push(
             Response::builder()
                 .status(StatusCode::BAD_REQUEST)
@@ -560,6 +604,28 @@ mod tests {
         let r = do_request().await.unwrap();
         assert_eq!(r.status(), StatusCode::OK);
 
+        // Should retry 429 Too Many Requests
+        mock.push(
+            Response::builder()
+                .status(StatusCode::TOO_MANY_REQUESTS)
+                .body(String::new())
+                .unwrap(),
+        );
+
+        let r = do_request().await.unwrap();
+        assert_eq!(r.status(), StatusCode::OK);
+
+        // Should retry 408 Request Timeout
+        mock.push(
+            Response::builder()
+                .status(StatusCode::REQUEST_TIMEOUT)
+                .body(String::new())
+                .unwrap(),
+        );
+
+        let r = do_request().await.unwrap();
+        assert_eq!(r.status(), StatusCode::OK);
+
         // Accepts 204 status code
         mock.push(
             Response::builder()
@@ -596,7 +662,7 @@ mod tests {
         assert_eq!(r.status(), StatusCode::OK);
 
         // Handles redirect loop
-        for _ in 0..10 {
+        for _ in 0..11 {
             mock.push(
                 Response::builder()
                     .status(StatusCode::FOUND)
@@ -618,7 +684,7 @@ mod tests {
         );
 
         let e = do_request().await.unwrap_err();
-        assert!(matches!(e.inner, RequestError::BareRedirect));
+        assert!(matches!(e.inner(), RequestError::BareRedirect));
         assert_eq!(e.inner().to_string(), "Received redirect without LOCATION, this normally indicates an incorrectly configured region");
 
         // Gives up after the retrying the specified number of times
@@ -631,26 +697,36 @@ mod tests {
             );
         }
 
-        let e = do_request().await.unwrap_err().to_string();
+        let e = do_request().await.unwrap_err();
         assert!(
-            e.contains(" after 2 retries, max_retries: 2, retry_timeout: 1000s  - Server returned non-2xx status code: 502 Bad Gateway"),
+            e.to_string().contains(" after 2 retries, max_retries: 2, retry_timeout: 1000s  - Server returned non-2xx status code: 502 Bad Gateway"),
             "{e}"
         );
+        // verify e.source() is available as well for users who need programmatic access
+        assert_eq!(
+            e.source().unwrap().to_string(),
+            "Server returned non-2xx status code: 502 Bad Gateway: ",
+        );
 
         // Panic results in an incomplete message error in the client
-        mock.push_fn(|_| panic!());
+        mock.push_fn::<_, String>(|_| panic!());
         let r = do_request().await.unwrap();
         assert_eq!(r.status(), StatusCode::OK);
 
         // Gives up after retrying multiple panics
         for _ in 0..=retry.max_retries {
-            mock.push_fn(|_| panic!());
+            mock.push_fn::<_, String>(|_| panic!());
         }
-        let e = do_request().await.unwrap_err().to_string();
+        let e = do_request().await.unwrap_err();
         assert!(
-            e.contains("after 2 retries, max_retries: 2, retry_timeout: 1000s  - HTTP error: error sending request"),
+            e.to_string().contains("after 2 retries, max_retries: 2, retry_timeout: 1000s  - HTTP error: error sending request"),
             "{e}"
         );
+        // verify e.source() is available as well for users who need programmatic access
+        assert_eq!(
+            e.source().unwrap().to_string(),
+            "HTTP error: error sending request",
+        );
 
         // Retries on client timeout
         mock.push_async_fn(|_| async move {
@@ -703,7 +779,7 @@ mod tests {
         assert!(!err.contains("SENSITIVE"), "{err}");
 
         for _ in 0..=retry.max_retries {
-            mock.push_fn(|_| panic!());
+            mock.push_fn::<_, String>(|_| panic!());
         }
 
         let req = client
@@ -751,4 +827,57 @@ mod tests {
         // Shutdown
         mock.shutdown().await
     }
+
+    #[tokio::test]
+    async fn test_connection_reset_is_retried() {
+        let retry = RetryConfig {
+            backoff: Default::default(),
+            max_retries: 2,
+            retry_timeout: Duration::from_secs(1),
+        };
+        assert!(retry.max_retries > 0);
+
+        // Setup server which resets a connection and then quits
+        let listener = TcpListener::bind("::1:0").await.unwrap();
+        let url = format!("http://{}", listener.local_addr().unwrap());
+        let handle = tokio::spawn(async move {
+            // Reset the connection on the first n-1 attempts
+            for _ in 0..retry.max_retries {
+                let (stream, _) = listener.accept().await.unwrap();
+                // TcpStream::set_linger is deprecated but this use case is valid to reset the stream
+                #[allow(deprecated)]
+                stream.set_linger(Some(Duration::from_secs(0))).unwrap();
+            }
+            // Succeed on the last attempt
+            let (stream, _) = listener.accept().await.unwrap();
+            http1::Builder::new()
+                // we want the connection to end after responding
+                .keep_alive(false)
+                .serve_connection(
+                    TokioIo::new(stream),
+                    service_fn(move |_req| async {
+                        Ok::<_, Infallible>(HttpResponse::new("Success!".to_string().into()))
+                    }),
+                )
+                .await
+                .unwrap();
+        });
+
+        // Perform the request
+        let client = HttpClient::new(reqwest::Client::new());
+        let ctx = &mut RetryContext::new(&retry);
+        let res = client
+            .get(url)
+            .retryable_request()
+            .send(ctx)
+            .await
+            .expect("request should eventually succeed");
+        assert_eq!(res.status(), StatusCode::OK);
+        assert!(ctx.exhausted());
+
+        // Wait for server to shutdown
+        let _ = timeout(Duration::from_secs(1), handle)
+            .await
+            .expect("shutdown shouldn't hang");
+    }
 }
